//+------------------------------------------------------------------+
//|                                                   my_Gapper2.mq4 |
//|                                  Copyright  2011, EarnForex.com |
//|                                         http://www.earnforex.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, EarnForex.com" 
#property link      "http://www.earnforex.com"

/*

   Tests the gaps for potential profit using entry against the gap's 
   direction. Gaps are entered at the week start, exited on Friday 
   (should wait for Friday end but that's not implemented here). Gap 
   is considered good if it's at least MinGap * Spread points wide. 
   Run it in Strategy Tester on daily timeframe.
   DO NOT USE FOR LIVE TRADING!

*/

extern int MinGap = 5; // Minimum gap size, in Spreads
extern double Lots      = 0.1;
extern int StopLoss     = 0;
extern int TakeProfit   = 0;
extern int Slippage     = 3;

int Magic;
//Depend on broker's quotes
double Poin;
int Deviation;

int LastBars = 0; //Amount of bars

//+------------------------------------------------------------------+ 
//| Custom indicator initialization function                       | 
//+------------------------------------------------------------------+ 
int init()
{
	Poin = Point;
	Deviation = Slippage;
	//Checking for unconvetional Point digits number
   if ((Point == 0.00001) || (Point == 0.001))
   {
      Poin *= 10;
      Deviation *= 10;
   }

   Magic = Period()+573495934;
   return(0);
}

//+------------------------------------------------------------------+ 
//| Custom indicator deinitialization function                       | 
//+------------------------------------------------------------------+ 
int deinit() 
{ 
   return(0); 
} 

//+------------------------------------------------------------------+ 
//| Enter if gap is detected on week's beginning. Exit on Friday.    | 
//+------------------------------------------------------------------+ 
int start() 
{ 
   if (DayOfWeek() == 5) ClosePrev(); // Close trades on Friday

   int spread = MarketInfo(Symbol(), MODE_SPREAD);

   if (LastBars < iBars(NULL, PERIOD_W1))
   {
      LastBars = iBars(NULL, PERIOD_W1);
  
      //Going against the gap's direction
      //Gap down
      if      (iClose(NULL, PERIOD_W1, 2) - iOpen(NULL, PERIOD_W1, 1) > spread * MinGap * Point) fBuy();
      //Gap up
      else if (iOpen(NULL, PERIOD_W1, 1) - iClose(NULL, PERIOD_W1, 2) > spread * MinGap * Point) fSell();
   }
   return(0);
}

//+------------------------------------------------------------------+
//| Sell                                                             |
//+------------------------------------------------------------------+
int fSell()
{
	double SL, TP;
	RefreshRates();
	if (StopLoss > 0) SL = Bid + StopLoss * Poin;
	else SL = 0;
   if (TakeProfit > 0) TP = Bid - TakeProfit * Poin;
	else TP = 0;	
	int result = OrderSend(Symbol(), OP_SELL, Lots, Bid, Deviation, SL, TP, "my_Gapper2", Magic);
	if (result == -1)
	{
		int e = GetLastError();
		Print(e);
	}
	else return(result);
}

//+------------------------------------------------------------------+
//| Buy                                                              |
//+------------------------------------------------------------------+
int fBuy()
{
	double SL, TP;
	RefreshRates();
	if (StopLoss > 0) SL = Ask - StopLoss * Poin;
	else SL = 0;
   if (TakeProfit > 0) TP = Ask + TakeProfit * Poin;
	else TP = 0;	
	int result = OrderSend(Symbol(), OP_BUY, Lots, Ask, Deviation, SL, TP, "my_Gapper2", Magic);
	if (result == -1)
	{
		int e = GetLastError();
		Print(e);
	}
	else return(result);
}

//+------------------------------------------------------------------+
//| Close previous position                                          |
//+------------------------------------------------------------------+
void ClosePrev()
{
   int total = OrdersTotal();
   for (int i = 0; i < total; i++)
   {
      if (OrderSelect(i, SELECT_BY_POS) == false) continue;
      if ((OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic))
      {
         if (OrderType() == OP_BUY)
         {
            RefreshRates();
            OrderClose(OrderTicket(), OrderLots(), Bid, Deviation);
         }
         else if (OrderType() == OP_SELL)
         {
            RefreshRates();
            OrderClose(OrderTicket(), OrderLots(), Ask, Deviation);
         }
      }
   }
}